# Copyright (C) 2006-2008 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from LuciZopePerm import userAuthenticated
from LuciDB import allowed_systems
from LuciSyslog import get_logger
from conga_constants import LUCI_DEBUG_MODE, INACTIVITY_TIMEOUT_SEC

luci_log = get_logger()

def siteIsSetup(self):
	import os
	from ricci_communicator import CERTS_DIR_PATH

	try:
		return os.path.isfile('%sprivkey.pem' % CERTS_DIR_PATH) and os.path.isfile('%scacert.pem' % CERTS_DIR_PATH)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('SIS0: %r %s' % (e, str(e)))
	return False

def strFilter(regex, replaceChar, arg):
	import re
	return re.sub(regex, replaceChar, arg)

# removes systems that user is not authorized access to
def get_systems_statuses(self, systems, from_cache=False):
	from HelperFunctions import get_system_info

	CACHED_INDEX = '_get_systems_statuses()_cached_result_'
	session = self.REQUEST.SESSION
	if session.has_key(CACHED_INDEX):
		res = session[CACHED_INDEX]
		if res is not None:
			session.set(CACHED_INDEX, None)
			if from_cache:
				return res

	allowed_sys_list = allowed_systems(self, systems)
	ss_list = get_system_info(self, allowed_sys_list)
	session.set(CACHED_INDEX, ss_list)
	return ss_list

def set_persistent_var(self, var_name, default_value):
	request = self.REQUEST
	response = request.RESPONSE
	session = request.SESSION

	# TODO: add username into cookie_prefix, so users don't overwrite each other
	cookie_prefix = '__luci_storage_cookie_'

	value = default_value
	if request.has_key(var_name):
		value = request[var_name]
	elif session.has_key(var_name):
		value = session[var_name]
	elif request.cookies.has_key(cookie_prefix + var_name):
		value = request.cookies[cookie_prefix + var_name]

	session.set(var_name, value)
	response.setCookie(cookie_prefix + var_name, value,
		expires='Tue, 30 Jun 2060 12:00:00 GMT')
	return value

# returns (str(float), units) that fits best,
# takes prefered units into account

def bytes_to_value_prefunits(self, bytes):
	from HelperFunctions import bytes_to_value_units, convert_bytes, get_units_multiplier

	p_units = self.REQUEST.SESSION.get('preferred_size_units')
	dummy, units = bytes_to_value_units(bytes)
	if get_units_multiplier(units) > get_units_multiplier(p_units):
		units = p_units
	return (convert_bytes(bytes, units), units)

def getTabs(self, req):
	if not userAuthenticated(self):
		return []

	enforce_inactivity_timeout(self, req)

	htab = {
		'Title': 'homebase',
		'Description': 'Home base for this luci server',
		'Taburl': '/luci/homebase',
		'isSelected': False
	}
	ctab = {
		'Title': 'cluster',
		'Description': 'Cluster configuration area',
		'Taburl': '/luci/cluster/index_html?pagetype=3',
		'isSelected': False
	}
	stab = {
		'Title': 'storage',
		'Description': 'Storage configuration page',
		'Taburl': '/luci/storage',
		'isSelected': False
	}

	try:
		baseurl = req['URL']
		if baseurl.find('cluster') > (-1):
			ctab['isSelected'] = True
		elif baseurl.find('storage') > (-1):
			stab['isSelected'] = True
		else:
			htab['isSelected'] = True
	except KeyError, e:
		pass
	except Exception, e:
		htab['isSelected'] = True

	return [ htab, ctab, stab ]

def appendModel(request, model):
	try:
		request.SESSION.set('model', model)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('Appending model to request failed: %r %s' \
				% (e, str(e)))

def GetReqVars(req, varlist):
	ret = {}
	from types import ListType

	for i in varlist:
		pval = None
		if req and req.has_key(i):
			pval = req[i]
			if type(req[i]) is not ListType:
				pval = req[i].strip()
			if not pval:
				pval = None
		if req and pval is None:
			if req.form and req.form.has_key(i):
				pval = req.form[i]
				if type(req.form[i]) is not ListType:
					pval.strip()
				if not pval:
					pval = None
		ret[i] = pval
	return ret

def GetFormVars(req, varlist):
	ret = {}
	from types import ListType

	for i in varlist:
		pval = None
		if req and req.has_key(i):
			pval = req[i]
			if type(req[i]) is not ListType:
				pval = req[i].strip()
			if not pval:
				pval = None
		ret[i] = pval
	return ret

def getClusterURL(self, request, model):
	try:
		clustername = request['clustername'].strip()
		if not clustername:
			raise Exception, 'cluster name from request is blank'
	except:
		try:
			clustername = model.getClusterName()
			if not clustername:
				raise Exception, 'cluster name from model is blank'
		except:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('GCURL0: unable to get cluster name')
			return ''

	return '/luci/cluster/index_html?pagetype=7&clustername=%s' % clustername

def user_update_inactive(self, request):
	from time import time
	if userAuthenticated(self):
		request.SESSION.set('last_activity', int(time()))
		request.SESSION.setTimeoutMinutes(max(INACTIVITY_TIMEOUT_SEC / 60, 10))

def user_check_inactive(self, request):
	from time import time

	if not userAuthenticated(self):
		return (False, -1)

	cur_time = int(time())
	old_time = request.SESSION.get('last_activity', None)
	if old_time is None:
		request.SESSION.set('last_activity', cur_time)
		request.SESSION.setTimeoutMinutes(max(INACTIVITY_TIMEOUT_SEC / 60, 10))
		return (False, -2)
	old_time = int(old_time)

	time_diff = cur_time - old_time
	if time_diff >= INACTIVITY_TIMEOUT_SEC:
		return (True, time_diff)
	return (False, time_diff)

def enforce_inactivity_timeout(self, request):
	from time import time
	pagetype = request.get('pagetype')
	if pagetype != '1010' and pagetype != '1011':
		(ret, dummy) = user_check_inactive(self, request)
		if ret is True:
			try:
				request.SESSION.getBrowserIdManager().flushBrowserIdCookie()
				request.SESSION.invalidate()
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('UCI0: %r %s' % (e, str(e)))
			request.RESPONSE.redirect('/luci/logout')
		else:
			request.SESSION.set('last_activity', int(time()))
